@@ -1,16 +1,43 @@
 from bespin.amazon.s3 import delete_key_from_s3, list_keys_from_s3_path, upload_file_to_s3
+from bespin.option_spec import stack_specs
 from bespin.errors import NoSuchStack
 from bespin.layers import Layers
 from bespin import helpers as hp
 
+from input_algorithms.meta import Meta
 import logging
+import json
 import os
 
 log = logging.getLogger("bespin.actions.builder")
 
 class Builder(object):
+    def sanity_check(self, stack, stacks, ignore_deps=False, checked=None):
+        """Check for missing environment variables in all the stacks"""
+        if checked is None:
+            checked = []
+
+        if stack.stack_name in checked:
+            return
+
+        log.info("Sanity checking %s", stack.key_name)
+        stack.find_missing_env()
+        stack_specs.stack_json_spec().normalise(Meta({}, []), stack.stack_json_obj)
+        if stack.params_json_obj:
+            stack_specs.params_json_spec().normalise(Meta({}, []), stack.params_json_obj)
+
+        if not ignore_deps and not stack.ignore_deps:
+            for dependency in stack.dependencies(stacks):
+                self.sanity_check(stacks[dependency], stacks, ignore_deps, checked + [stack.stack_name])
+
+        if any(stack.build_after):
+            for dependency in stack.build_after:
+                self.sanity_check(stacks[dependency], stacks, ignore_deps, checked + [stack.stack_name])
+
     def deploy_stack(self, stack, stacks, credentials, made=None, ignore_deps=False):
         """Make us an stack"""
+        self.sanity_check(stack, stacks, ignore_deps=ignore_deps)
+
         made = made or {}
 
         if stack.name in made:
@@ -21,7 +48,10 @@ class Builder(object):
 
         if not ignore_deps and not stack.ignore_deps:
             for dependency in stack.dependencies(stacks):
-                self.deploy_stack(stacks[dependency], stacks, made=made, ignore_deps=True)
+                self.deploy_stack(stacks[dependency], stacks, credentials, made=made, ignore_deps=True)
+
+            for dependency in stack.dependencies(stacks):
+                stacks[dependency].cloudformation.wait()
 
         # Should have all our dependencies now
         log.info("Making stack for '%s' (%s)", stack.name, stack.stack_name)
@@ -29,8 +59,14 @@ class Builder(object):
         made[stack.name] = True
 
         if any(stack.build_after):
+            stack.cloudformation.wait()
             for dependency in stack.build_after:
-                self.deploy_stack(stacks[dependency], stacks, made=made, ignore_deps=True)
+                self.deploy_stack(stacks[dependency], stacks, credentials, made=made, ignore_deps=True)
+
+            for dependency in stack.build_after:
+                stacks[dependency].cloudformation.wait()
+
+        stack.cloudformation.wait()
 
     def layered(self, stacks, only_pushable=False):
         """Yield layers of stacks"""
@@ -45,7 +81,11 @@ class Builder(object):
 
     def build_stack(self, stack):
         print("Building - {0}".format(stack.stack_name))
-        print(stack.params_json_obj)
+        print(json.dumps(stack.params_json_obj, indent=4))
+        if stack.skip_update_if_equivalent and all(check.resolve() for check in stack.skip_update_if_equivalent):
+            log.info("Stack is determined to be the same, not updating")
+        else:
+            stack.create_or_update()
 
     def find_missing_build_env(self, stack):
         for artifact in stack.artifacts.values():
